#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import os.path
import io

from googleapiclient.http import MediaIoBaseDownload
from zdrive import DriveAPI
from multiprocessing import Process


class Downloader(DriveAPI):
    def __init__(self):
        super(Downloader, self).__init__()
        self.DEFAULT_STORAGE_PATH = 'drive_content'

    def __create_Directory(self, path=None):
        if path is None:
            path = self.DEFAULT_STORAGE_PATH
        if not os.path.isdir(path):
            os.mkdir(path=path)

        return path

    def downloadFolder(self, folderId, destinationFolder=None):

        path = self.__create_Directory(destinationFolder)
        page_token = None

        while True:
            results = self.service.files().list(
                spaces='drive',
                pageToken=page_token,
                q="parents in '{0}'".format(folderId),
                fields="nextPageToken, files(id, name, mimeType)"
            ).execute()

            items = results.get('files', [])
            download_processes = []
            for item in items:
                itemName = item['name']
                itemId = item['id']
                itemType = item['mimeType']
                filePath = os.path.join(path, itemName)

                if itemType == 'application/vnd.google-apps.folder':
                    print("Stepping into folder: {0}".format(filePath))
                    self.downloadFolder(itemId, filePath)  # Recursive call
                else:
                    p = Process(target=self.downloadFile, args=[itemId, filePath])
                    p.start()
                    download_processes.append(p)

            for process in download_processes:
                process.join()

            page_token = results.get('nextPageToken', None)
            if page_token is None:
                break

    def downloadFile(self, fileId, filePath=None):
        # Note: The parent folders in filePath must exist
        print("\t -> Downloading file with id: {0} name: {1}".format(fileId, filePath))
        request = self.service.files().get_media(fileId=fileId)
        fh = io.FileIO(filePath, mode='wb')

        try:
            downloader = MediaIoBaseDownload(fh, request, chunksize=1024 * 1024)

            current_progress = -1
            done = False
            while done is False:
                status, done = downloader.next_chunk(num_retries=2)
                if status:
                    progress = int(status.progress() * 100)
                    if progress % 10 == 0 and progress != current_progress:
                        print(
                            " \t Downloading fileId: {0}, {1}% complete!".format(fileId, int(status.progress() * 100)))
                        current_progress = progress
            print("Download Complete!")
        except Exception as e:
            print(str(e))
            return
        finally:
            fh.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import json
import os.path

from mimetypes import MimeTypes
from googleapiclient.http import MediaFileUpload
from zdrive import DriveAPI


class Uploader(DriveAPI):
    def __init__(self):
        super(Uploader, self).__init__()
        self.__default_depth = 5

    @staticmethod
    def __isValidDir(path):
        """
        checks whether the given path
        exists in the directory or not
        :type path: string
        :param path: the path to check
        :return: 1 if path exists, 0 if it doesn't
        """

        return os.path.isdir(path)

    def createFolder(self, name, parentId=None):
        try:
            file_metadata = {
                'name': name,
                'mimeType': 'application/vnd.google-apps.folder'
            }
            if parentId is not None:
                file_metadata["parents"] = [parentId]

            file = self.service.files().create(body=file_metadata,
                                               fields='id').execute()

            folder_id = file.get('id')

            print('\t -> Created folder: {0} with ID: {1}'.format(name, folder_id))
            return folder_id

        except Exception as e:
            raise Exception(e)

    def uploadFile(self, filePath, parentId):
        try:
            file_name = os.path.basename(filePath)
            mimetype = MimeTypes().guess_type(file_name)[0]

            file_metadata = {'name': file_name}

            if parentId is not None:
                file_metadata["parents"] = [parentId]

            media = MediaFileUpload(filePath,
                                    mimetype=mimetype,
                                    resumable=True)

            file = self.service.files().create(body=file_metadata,
                                               media_body=media,
                                               fields='id').execute()

            file_id = file.get('id')
            print('\t -> File {0} uploaded successfully! ID: {1}'.format(file_name, file_id))
            return file_id
        except Exception as e:
            raise Exception(e)

    def __upload_folder(self, folder_path, current_depth, max_depth, parent_id=None):
        try:
            # check depth
            if current_depth > max_depth:
                return {}

            print("Uploading folder {}".format(folder_path))

            # get files and folders
            root_files = []
            root_folders = []
            for (_, dirs, files) in os.walk(folder_path, topdown=True):
                root_files.extend(files)
                root_folders.extend(dirs)
                break

            # upload files
            files_upload_dict = self.__upload_files(folder_path, root_files, parent_id)

            # recursively create and upload folder
            folders_upload_dict = {}

            current_depth += 1
            for folder in root_folders:
                folder_id = self.createFolder(folder, parent_id)
                full_path = os.path.join(folder_path, folder)
                print("\t -> Stepping inside folder {0}".format(full_path))
                folder_result_dict = self.__upload_folder(full_path, current_depth, max_depth, folder_id)
                folder_result_dict["id"] = folder_id
                folders_upload_dict[full_path] = folder_result_dict

            print("Success! Uploaded folder {} ".format(folder_path))

            result = {
                'files': files_upload_dict,
                'folders': folders_upload_dict
            }

            return result
        except Exception as e:
            raise Exception(e)

    def uploadFolder(self, folderPath, max_depth=None, parentId=None):
        try:
            if not self.__isValidDir(folderPath):
                raise Exception("Not a valid folder path {0}".format(folderPath))

            if max_depth is None:
                max_depth = self.__default_depth

            upload_status = self.__upload_folder(folderPath, 1, max_depth, parentId)
            return json.dumps(upload_status, indent=4)
        except Exception as e:
            raise Exception(e)

    def __upload_files(self, folder_path, root_files, parent_id=None):
        try:
            if len(root_files) == 0:
                return {}
            uploaded_files = {}
            for file in root_files:
                file_path = os.path.join(folder_path, file)
                file_id = self.uploadFile(file_path, parent_id)
                uploaded_files[file_path] = file_id
            return uploaded_files
        except Exception as e:
            print(str(e))
            return {}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
import pickle
import os.path

from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request


class DriveAPI:
    global SCOPES

    # Define the scopes
    SCOPES = ['https://www.googleapis.com/auth/drive']

    def __init__(self):

        # Variable self.creds will
        # store the user access token.
        # If no valid token found
        # we will create one.
        self.creds = None

        # The file token.pickle stores the
        # user's access and refresh tokens. It is
        # created automatically when the authorization
        # flow completes for the first time.

        # Check if file token.pickle exists
        if os.path.exists('token.pickle'):
            # Read the token from the file and
            # store it in the variable self.creds
            with open('token.pickle', 'rb') as token:
                self.creds = pickle.load(token)

        # If no valid credentials are available,
        # request the user to log in.
        if not self.creds or not self.creds.valid:

            # If token is expired, it will be refreshed,
            # else, we will request a new one.
            if self.creds and self.creds.expired and self.creds.refresh_token:
                self.creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    "credentials.json", SCOPES)
                self.creds = flow.run_local_server(port=0)

            # Save the access token in token.pickle
            # file for future usage
            with open('token.pickle', 'wb') as token:
                pickle.dump(self.creds, token)

        # Connect to the API service
        self.service = build('drive', 'v3', credentials=self.creds)

    def listFolders(self):
        try:
            page_token = None
            folders = []
            while True:
                response = self.service.files().list(q="mimeType='application/vnd.google-apps.folder'",
                                                     spaces='drive',
                                                     fields='nextPageToken, files(id, name)',
                                                     pageToken=page_token).execute()

                for file in response.get('files', []):
                    # Process change
                    folder_data = (file.get('name'), file.get('id'))
                    folders.append(folder_data)

                page_token = response.get('nextPageToken', None)
                if page_token is None:
                    break

            return folders
        except Exception as e:
            print(str(e))
            return None
